@@ -98,6 +98,15 @@ get_options <- function() {
   }
 }
 
+#' Gets the currently configured GoFigr client
+#'
+#' @returns GoFigr client
+#' @export
+get_client <- function() {
+  opts <- get_options()
+  return(opts$client)
+}
+
 is_debug_on <- function() {
   opts <- get_options()
   if(is.null(opts)) {
@@ -305,6 +314,62 @@ display <- function(rev, plot_obj) {
   }
 }
 
+
+#' Executes an expression while isolating any new graphics devices it creates.
+#'
+#' @param expr The R expression to evaluate.
+#' @return result of evaluating expr
+#' @export
+with_isolated_devices <- function(expr) {
+  # 1. Backup the original device state
+  original_devices <- grDevices::dev.list()
+  original_active_device <- tryCatch(grDevices::dev.cur(), error = function(e) NULL)
+
+  # 2. Schedule the cleanup to run when the function exits (even on error)
+  on.exit({
+    # Get the list of devices after the expression has run
+    new_devices <- grDevices::dev.list()
+
+    # Identify any devices that were opened by the expression
+    devices_to_close <- setdiff(new_devices, original_devices)
+
+    # Close them
+    for (dev in devices_to_close) {
+      grDevices::dev.off(dev)
+    }
+
+    # Restore the originally active device, if it still exists
+    if (!is.null(original_active_device) && original_active_device %in% grDevices::dev.list()) {
+      grDevices::dev.set(original_active_device)
+    }
+  })
+
+  # 3. Evaluate the user's expression
+  force(expr)
+}
+
+
+#' Tries to convert expression to a grob, returning it unchanged if it fails.
+#'
+#' @param expr expression/object to convert
+#'
+#' @returns grob if successful, expr if not
+#' @export
+try_base2grob <- function(expr) {
+  as_gg <- NULL
+
+  tryCatch({
+    with_isolated_devices({
+      as_gg <- ggplotify::as.ggplot(ggplotify::base2grob(function() {
+        expr
+    }))})
+  }, error=function(err) {
+    as_gg <<- expr
+  })
+
+  return(as_gg)
+}
+
 #' Captures output from grid graphics (ggplot2, lattice, ComplexHeatmap, etc.)
 #' and publishes it to GoFigr.
 #'
@@ -314,11 +379,15 @@ display <- function(rev, plot_obj) {
 #' @return GoFigr Revision object
 #' @export
 publish_base <- function(expr, ...) {
-  asgg <- ggplotify::as.ggplot(ggplotify::base2grob(function() {
-    expr
-  }))
+  .Deprecated("publish")
 
-  publish(asgg, ...)
+  with_isolated_devices({
+    asgg <- ggplotify::as.ggplot(ggplotify::base2grob(function() {
+        expr
+      }))
+
+    publish(asgg, ..., base_convert=FALSE)
+  })
 }
 
 
@@ -333,6 +402,7 @@ publish_base <- function(expr, ...) {
 #' @param data optional data to save with this figure. The data will be saved as RDS.
 #' @param metadata optional metadata
 #' @param show whether to display the figure after publication
+#' @param base_convert whether to try converting base graphics to grid graphics
 #'
 #' @returns GoFigr revision object
 #' @export
@@ -344,14 +414,21 @@ publish <- function(plot_obj,
                     image_formats=c("eps"),
                     data=NULL,
                     metadata=NULL,
-                    show=TRUE) {
-
+                    show=TRUE,
+                    base_convert=TRUE) {
   gf_opts <- get_options()
   if(is.null(gf_opts)) {
     warning("GoFigr hasn't been configured. Did you call gofigR::enable()?")
     return(invisible(NULL))
   }
 
+  if(base_convert) {
+    res <- try_base2grob(plot_obj)
+    if(is_supported(res)) {
+      plot_obj <- res
+    }
+  }
+
   plot_obj <- to_ggplot(plot_obj)
   if(is.null(plot_obj)) {
     warning(paste0("Provided object cannot be converted to ggplot: ", plot_obj))
@@ -363,7 +440,10 @@ publish <- function(plot_obj,
   input_path <- first_valid(input_path, context$input_path)
   input_contents <- first_valid(input_contents, context$input_contents)
   chunk_code <- first_valid(chunk_code, context$chunk_code)
-  metadata <- first_valid(metadata, context$metadata)
+
+  if(is.null(metadata)) {
+    metadata <- context$metadata
+  }
 
   if(is.null(figure_name)) {
     figure_name <- "Anonymous Figure"
@@ -404,7 +484,9 @@ publish <- function(plot_obj,
 
   other_data <- annotate(rev_bare, plot_obj, fig$name, input_path, input_contents, chunk_code, data)
 
-  rev <- gofigR::update_revision_data(client, rev_bare, silent=TRUE, new_data=append(image_data, other_data))
+  rev <- gofigR::update_revision_data(client, rev_bare, silent=TRUE,
+                                      new_data=append(image_data, other_data),
+                                      assets=unname(gf_opts$assets))
   file.remove(png_path)
 
   if(gf_opts$verbose) {
@@ -417,19 +499,44 @@ publish <- function(plot_obj,
     display(rev, plot_obj)
   }
 
-  return(invisible(rev))
+  class(rev) <- "gofigr_revision"
+  rev$client <- gf_opts$client
+
+  return(rev)
 }
 
-to_ggplot <- function(x, warn=FALSE) {
-  if(is_ggplot(x)) {
+#' Default print method for GoFigr revisions.
+#'
+#' @param x revision
+#' @param ... passed to base::print
+#'
+#' @returns NA
+#' @export
+print.gofigr_revision <- function(x, ...) {
+  print(get_revision_url(x), ...)
+}
+
+#' Default cat method for GoFigr revisions.
+#'
+#' @param x revision
+#' @param ... passed to cat
+#'
+#' @returns NA
+#' @export
+cat.gofigr_revision <- function(x, ...) {
+  cat(paste0(get_revision_url(x), "\n"), ...)
+}
+
+to_ggplot <- function(x, warn = FALSE) {
+  if (is_ggplot(x)) {
     return(x)
   }
 
   converted <- NULL
   tryCatch({
     converted <- ggplotify::as.ggplot(x)
-  }, error=function(err) {
-    if(warn) {
+  }, error = function(err) {
+    if (warn) {
       warning(err)
     }
     converted <- NULL
@@ -466,11 +573,25 @@ gf_plot <- make_invisible(intercept(base::plot))
 gf_print <- make_invisible(intercept(base::print))
 
 intercept_base <- function(env=.GlobalEnv) {
-  assign("plot", gf_plot, env)
   assign("print", gf_print, env)
 }
 
-#' Enables GoFigr support.
+
+#' Syncs a file with the GoFigr service and stores a reference. The file
+#' will be associated with all figures published after this call.
+#'
+#' @param path path to the file
+#'
+#' @returns null
+#' @export
+sync_file <- function(path) {
+  opts <- get_options()
+  asset_rev <- sync_workspace_asset(opts$client, opts$workspace, path)
+  opts$assets[[get_api_id(asset_rev)]] <- asset_rev
+}
+
+
+#' Enables GoFigr in the current R/Rmd file.
 #'
 #' @param auto_publish will publish all plots automatically if TRUE. Note
 #'  that setting this option will re-assign plot() and print() in the global environment.
@@ -485,6 +606,7 @@ intercept_base <- function(env=.GlobalEnv) {
 #' @param show which figure to display in the document: original, watermark, or hide. Note that this setting \
 #' only affects the display and doesn't change what gets published: e.g. even if you choose to display \
 #' the original figure, the watermarked version will still be published to GoFigr.
+#' @param api_key GoFigr API key
 #'
 #' @return named list of GoFigr options
 #' @export
@@ -497,6 +619,7 @@ enable <- function(auto_publish=FALSE,
                    watermark=QR_WATERMARK,
                    verbose=FALSE,
                    debug=FALSE,
+                   api_key=NULL,
                    show="watermark") {
   check_show_setting(show)
 
@@ -507,7 +630,9 @@ enable <- function(auto_publish=FALSE,
   }
 
   # Create the GoFigr client
-  gf <- gofigr_client(workspace=workspace, verbose=verbose)
+  gf <- gofigr_client(workspace = workspace,
+                      verbose = verbose,
+                      api_key = api_key)
 
   # Find the analysis
   if(!is.null(analysis_api_id)) {
@@ -529,6 +654,7 @@ enable <- function(auto_publish=FALSE,
     verbose <- verbose
     debug <- debug
     show <- show
+    assets <- list()
 
     return(environment())
   })))
